from ..utils import Scraper
from bs4 import BeautifulSoup
from collections import OrderedDict
import json
import os
import requests


class UTMTimetable:

    host = 'https://student.utm.utoronto.ca/timetable/'

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('UTMTimetable initialized.')
        Scraper.logger.info('Not implemented.')
        Scraper.logger.info('UTMTimetable completed.')



from ..utils import Scraper
from bs4 import BeautifulSoup
from collections import OrderedDict
import re
import tidylib


class UTSGTimetable:

    host = 'http://www.artsandscience.utoronto.ca/ofr/timetable/'
    day_map = {
        'M': 'MONDAY',
        'T': 'TUESDAY',
        'W': 'WEDNESDAY',
        'R': 'THURSDAY',
        'F': 'FRIDAY',
        'S': 'SATURDAY'
    }
    terms = ['summer', 'winter']

    @staticmethod
    def scrape(location):
        """Update the local JSON files for this scraper."""

        Scraper.logger.info('UTSGTimetable initialized.')

        for term in UTSGTimetable.terms:
            year = 0
            data = UTSGTimetable.get_sponsors(term)
            sponsors = data['sponsors']

            if term == 'summer':
                # 2015 summer counts as 2014 term
                year = data['year'] - 1
            else:
                year = data['year']

            for sponsor in sponsors:
                Scraper.logger.info('Scraping %s/%s.' % (
                    term,
                    sponsor.split('.')[0]
                ))

                html = Scraper.get_html('%s/%s/%s' % (
                    UTSGTimetable.host,
                    term,
                    sponsor
                ))

                data = UTSGTimetable.parse_sponsor(html, year, term, sponsor)

                for course in data:
                    Scraper.save_json(course, location, course['id'])

        Scraper.logger.info('UTSGTimetable completed.')

    @staticmethod
    def parse_sponsor(html, year, term, sponsor=''):
        document, errors = \
            tidylib.tidy_document(html, options={'numeric-entities': 1})

        soup = BeautifulSoup(document, 'html.parser')

        table = soup.table

        trs = table.find_all('tr')[2:] \
            if 'assem' in sponsor else table.find_all('tr')[2:]

        course_info = []
        current_course = []
        current_code = ''
        current_breadths = []
        current_section = None

        for tr in trs:
            if 'Cancel' in tr.get_text():
                continue

            tds = tr.find_all('td')

            i = 0
            while i < len(tds):
                colspan = tds[i].get('colspan')
                if colspan is None:
                    i += 1
                    continue
                else:
                    colspan = int(colspan) - 1
                    if colspan > 0:
                        for x in range(colspan):
                            tds.insert(i + 1, soup.new_tag('td'))
                            i += 1

            if len(tds) >= 9:
                course_code = UTSGTimetable.format_data(tds[0].get_text(),
                    '([A-Z]{3}[0-9]{3}[HY]1)')

                if len(course_code) > 0:
                    course_info.append(current_course)

                    current_course = [
                        None,            # course code
                        None,            # name
                        OrderedDict([]), # sections
                        []
                    ]

                    semester = tds[1].get_text().strip()
                    current_course[0] = course_code + semester

                    name = tds[2].get_text().split('\n')[0].strip()
                    current_course[1] = name

                section = UTSGTimetable.format_data(tds[3].get_text(),
                    '([LTP][0-9]{4})')

                if len(section) > 0:
                    current_section = section

                time = UTSGTimetable.format_data(tds[5].get_text(),
                    '([MTWRFS]{1,3}[0-9]{1,2}(?::[0-9]' +
                    '{2})?(?:-[0-9]{1,2}(?::[0-9]{2})?)?)')

                location = UTSGTimetable.format_data(tds[6].get_text(),
                    '([A-Z]{2,4}[ ]?[0-9]{1,8})')

                instructors = tds[7].get_text().strip()

                if instructors.lower() == 'tba':
                    instructors = ''

                if len(instructors) > 0:
                    instructors = instructors.split('/')
                    for i in range(len(instructors)):
                        instructors[i] = ' '.join([x.strip() for x in instructors[i].split('.')])
                else:
                    instructors = []

                try:
                    if not isinstance(current_course[2][current_section], list):
                        current_course[2][current_section] = []
                except KeyError:
                    current_course[2][current_section] = []

                current_course[2][current_section].append({
                    'time': time,
                    'location': location,
                    'instructors': instructors
                })

            elif len(tds) == 6:
                if tds[0].get('colspan') == '6':
                    course_code = \
                        UTSGTimetable.format_data(tds[0].get_text(),
                            '([A-Z]{3}[0-9]{3}[HY]{1}1[YFS]{1})')
                    breadths = [int(x) for x in
                                re.findall('(?:\()([12345])(?:\))',
                                tds[0].get_text().strip())]
                    name = ''.join(tds[0].get_text()
                        .replace('Categories ', ':')
                        .replace('Categories:', ':')
                        .split(':')[1:]).split(', Count')[0].strip()

                    if len(course_code) > 0:
                        course_info.append(current_course)
                        current_course = [
                            course_code,     # course code
                            name,            # name
                            OrderedDict([]), # sections
                            breadths
                        ]
                else:
                    section = UTSGTimetable.format_data(tds[0].get_text(),
                        '([LTP][0-9]{4})')

                    if len(section) > 0:
                        current_section = section

                    time = UTSGTimetable.format_data(tds[3].get_text(),
                        '([MTWRFS]{1,3}[0-9]{1,2}' +
                        '(?::[0-9]{2})?(?:-[0-9]{1,2}' +
                        '(?::[0-9]{2})?)?)')

                    location = UTSGTimetable.format_data(tds[4].get_text(),
                        '([A-Z]{2,4}[ ]?[0-9]{1,8})')

                    instructors = tds[5].get_text().strip()

                    if instructors.lower() == 'tba':
                        instructors = ''
                    else:
                        instructors = instructors.replace('.', '')

                    if len(instructors) > 0:
                        instructors = instructors.split('/')
                    else:
                        instructors = []

                    try:
                        if not isinstance(current_course[2][current_section],
                            list):
                            current_course[2][current_section] = []
                    except KeyError:
                        current_course[2][current_section] = []

                    current_course[2][current_section].append({
                        'time': time,
                        'location': location,
                        'instructors': instructors
                    })

        course_info = course_info[1:] + [current_course]

        courses = []

        for course in course_info:
            if len(course) == 0:
                continue

            course_id = course[0]
            course_term = ''

            if term == 'winter':
                if course[0][-1] in 'FY':
                    course_id += str(year) + '9'
                    course_term = str(year) + ' Fall'
                    if course[0][-1] == 'Y':
                        course_term += ' +'
                elif course[0][-1] == 'S':
                    course_id += str(year + 1) + '1'
                    course_term = str(year + 1) + ' Winter'
            elif term == 'summer':
                if course[0][-1] == 'Y':
                    course_id += str(year + 1) + '5'
                    course_term = str(year + 1) + ' Summer Y'
                elif course[0][-1] in 'FS':
                    course_id += str(year + 1) + '5' + course[0][-1]
                    if course_id[0][-1] == 'F':
                        course_term = str(year + 1) + ' Summer F'
                    elif course_id[0][-1] == 'S':
                        course_term = str(year + 1) + ' Summer S'

            course_code = course[0]

            level = int(course_code[3] + '00')

            sections = []
            for k in course[2]:
                code = k
                instructors = []
                time_data = []

                for x in course[2][k]:
                    if x['time'] != '':
                        instructors += x['instructors']

                        location = x['location']

                        # magic

                        days = re.findall('[MTWRFS]', x['time'])

                        time = ''
                        mob = re.search('\d', x['time'])
                        if mob:
                            time = x['time'][mob.start():]

                        hours = []
                        time = time.split('-')
                        for t in time:
                            if ':' in t:
                                t = t.split(':')
                                t = int(t[0]) + (int(t[1]) / 60)
                            else:
                                t = int(t)
                            if t >= 1 and t <= 8:
                                t += 12
                            hours.append(t)

                        if len(hours) == 1:
                            hours.append(hours[0] + 1)

                        for i in range(len(hours)):
                            if 1 <= hours[i] <= 8:
                                hours[i] += 12

                        for day_key in days:
                            day = UTSGTimetable.day_map[day_key]

                            time_data.append(OrderedDict([
                                ('day', day),
                                ('start', hours[0]),
                                ('end', hours[1]),
                                ('duration', hours[1] - hours[0]),
                                ('location', location)
                            ]))

                instructors = list(set(instructors))

                course[2][k]
                data = OrderedDict([
                    ('code', code),
                    ('instructors', instructors),
                    ('times', time_data)
                ])

                sections.append(data)

            course = OrderedDict([
                ('id', course_id),
                ('code', course_code),
                ('level', level),
                ('campus', 'UTSG'),
                ('term', course_term),
                ('meeting_sections', sections)
            ])

            courses.append(course)

        return courses

    @staticmethod
    def format_data(text, regex):
        text = re.search(regex, text.strip())
        if text is None:
            text = ''
        else:
            text = text.group(1)
        return text

    @staticmethod
    def get_sponsors(term):
        html = Scraper.get_html('%s/%s/index.html' % (
            UTSGTimetable.host,
            term
        ))

        soup = BeautifulSoup(html, 'html.parser')

        title = soup.title.get_text().strip()

        year = -1
        year = int(re.search('([0-9]{4})', title).group(0))

        sponsors = []
        for x in soup.find_all('a'):
            url = x.get('href')
            if '.htm' in url and '/' not in url:
                if 'intensive' not in url:
                    sponsors.append(url)

        return {
            'year': year,
            'sponsors': sponsors
        }



from ..utils import Scraper
from .utsg import UTSGExams
from .utm import UTMExams
from .utsc import UTSCExams
from bs4 import BeautifulSoup
from collections import OrderedDict
import json
import os
import requests


class Exams:

    host = 'http://www.artsandscience.utoronto.ca/ofr/calendar/'

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('Exams initialized.')
        UTSGExams.scrape(location)
        UTMExams.scrape(location)
        UTSCExams.scrape(location)
        Scraper.logger.info('Exams completed.')



from ..utils import Scraper
from bs4 import BeautifulSoup
from datetime import datetime
from collections import OrderedDict
import requests
import pytz


class UTSCExams:
    """A scraper for UTSC exams."""

    host = 'http://www.utsc.utoronto.ca/registrar/examination-schedule'
    s = requests.Session()

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('UTSCExams initialized.')

        exams = OrderedDict()

        headers = {
            'Referer': UTSCExams.host
        }
        html = UTSCExams.s.get('%s' % UTSCExams.host, headers=headers).text
        soup = BeautifulSoup(html, 'html.parser')

        for table in soup.find_all('table', class_='views-table'):
            for tr in table.find_all('tr')[1:]:
                data = [x.text.strip() for x in tr.find_all('td')]

                course_code = data[0]
                if ' ' in course_code:
                    course_code, lecture_code = course_code.split(' ')
                else:
                    lecture_code = None

                date = data[1]
                start, end = UTSCExams.parse_time(data[2], data[3], date)
                location_ = data[4]

                id_, course_id = UTSCExams.get_course_id(course_code, date)

                period = UTSCExams.get_period(date)

                if not id_ or not period:
                    continue

                doc = OrderedDict([
                    ('id', id_),
                    ('course_id', course_id),
                    ('course_code', course_code),
                    ('period', period),
                    ('date', date),
                    ('start_time', start),
                    ('end_time', end),
                    ('sections', [])
                ])

                if id_ not in exams:
                    exams[id_] = doc

                exams[id_]['sections'].append({
                    'section': lecture_code or '',
                    'location': location_
                })

        for id_, doc in exams.items():
            Scraper.save_json(doc, location, id_)

        Scraper.logger.info('UTSCExams completed.')

    @staticmethod
    def get_period(d):
        def get_date(month, date, year):
            months = {
                'dec': 12,
                'apr': 4,
                'june': 6,
                'aug': 8
            }
            return datetime.strptime('%s-%d-%d' % (year, months[month], date),
                                     '%Y-%m-%d')

        d = datetime.strptime(d, '%Y-%m-%d')

        year = d.year
        month = None

        for m, ld in (('dec', 31), ('apr', 30), ('june', 30), ('aug', 31)):
            if get_date(m, 1, year) <= d <= get_date(m, ld, year):
                month = m
                break

        if month:
            return '%s%s' % (month.upper(), str(year)[2:])

    @staticmethod
    def get_course_id(course_code, date):
        d = datetime.strptime(date, '%Y-%m-%d')
        month, year, period = d.strftime("%b").lower(), d.year, UTSCExams.get_period(date)
        endings = {
            'dec': {
                'F': '%s9' % str(year),
                'Y': '%s9' % str(int(year) - 1)
            },
            'apr': {
                'S': '%s1' % str(year),
                'Y': '%s9' % str(int(year) - 1)
            },
            'june': {
                'F': '%s5F' % str(year),
                'Y': '%s5' % str(year)
            },
            'aug': {
                'S': '%s5S' % str(year),
                'Y': '%s5' % str(year)
            }
        }

        season = course_code[-1]
        exam_id = course_id = None

        if month in endings and season in endings[month]:
            course_id = '%s%s' % (course_code, endings[month][season])
            exam_id = '%s%s' % (course_id, period)

        return exam_id, course_id

    @staticmethod
    def parse_time(start, end, date):
        def convert_time(t):
            h, m = [int(x) for x in t.split(':')]
            d = datetime.strptime('%s %s %s' % (date, h, m), '%Y-%m-%d %H %M')
            return d.replace(tzinfo=pytz.timezone('US/Eastern')).isoformat()
        return convert_time(start), convert_time(end)



from ..utils import Scraper
from .utm import UTMCalendar
from .utsc import UTSCCalendar
from .utsg import UTSGCalendar
from bs4 import BeautifulSoup
from collections import OrderedDict
import json
import os
import requests


class Calendar:

    host = 'http://www.artsandscience.utoronto.ca/ofr/calendar/'

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('Calendar initialized.')
        UTSGCalendar.scrape(location)
        UTMCalendar.scrape(location)
        UTSCCalendar.scrape(location)
        Scraper.logger.info('Calendar completed.')



from ..utils import Scraper, LayersScraper
from .utm import UTMTimetable
from .utsc import UTSCTimetable
from .utsg import UTSGTimetable
from bs4 import BeautifulSoup
from collections import OrderedDict
import json
import os
import requests


class Timetable:

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('Timetable initialized.')
        UTSGTimetable.scrape(location)
        UTMTimetable.scrape(location)
        UTSCTimetable.scrape(location)
        Scraper.logger.info('Timetable completed.')



import json
import logging
import os
import requests
import shutil
import sys


class Scraper:
    """Scraper class."""

    logger = logging.getLogger('uoftscrapers')
    s = requests.Session()

    @staticmethod
    def ensure_location(location):
        """Ensure that the location given exists."""

        if not os.path.exists(location):
            os.makedirs(location)

    @staticmethod
    def save_json(data, location, filename):
        Scraper.ensure_location(location)

        with open('%s/%s.json' % (location, filename),'w+') as outfile:
            json.dump(data, outfile)

    @staticmethod
    def get_html(url, params=None, cookies=None, headers=None, max_attempts=10):
        """Fetches the HTML page source, automatically retrying if it times out."""

        html = None
        attempts = 0
        while html is None and attempts < max_attempts:
            try:
                r = Scraper.s.get(url, params=params, cookies=cookies, headers=headers)
                if r.status_code == 200:
                    html = r.text
            except (requests.exceptions.Timeout,
                    requests.exceptions.ConnectionError):
                attempts += 1
                continue

        return html.encode('utf-8') if html else None

    @staticmethod
    def flush_percentage(decimal):
        """Update the last line in stdout to a percentage formatted value."""

        sys.stdout.write('%.2f%%\r' % (decimal * 100))
        sys.stdout.flush()

    @staticmethod
    def get_text_from_class(soup, name):
        obj = soup.find(class_=name)
        if obj != None:
            return obj.get_text().replace('\xa0', ' ').strip()
        else:
            return ''



from ..utils import Scraper, LayersScraper
from bs4 import BeautifulSoup
from collections import OrderedDict
import requests


class Food:
    """A scraper for UofT restaurants.

    UofT Food data is located at http://map.utoronto.ca
    """

    host = 'http://map.utoronto.ca/'
    campuses = [('utsg', 2), ('utm', 1), ('utsc', 0)]
    s = requests.Session()

    @staticmethod
    def scrape(location='.'):
        """Update the local JSON files for this scraper."""

        Scraper.logger.info('Food initialized.')

        for campus, food_index in Food.campuses:
            data = LayersScraper.get_layers_json(campus)[food_index]

            for entry in data['markers']:
                id_ = str(entry['id']).zfill(4)
                name = entry['title']

                building_id = LayersScraper.get_value(entry, 'building_code')

                address = ' '.join(filter(None,
                    LayersScraper.get_value(entry, 'address').split()))

                hours = Food.get_hours(id_)
                short_name = LayersScraper.get_value(entry, 'slug')

                desc = BeautifulSoup(
                    LayersScraper.get_value(entry, 'desc').strip(),
                    'html.parser').text

                tags = list(filter(None,
                    LayersScraper.get_value(entry, 'tags').lower().split(', ')))

                image = LayersScraper.get_value(entry, 'image')
                lat = LayersScraper.get_value(entry, 'lat', True)
                lng = LayersScraper.get_value(entry, 'lng', True)
                url = LayersScraper.get_value(entry, 'url')

                if not image == '':
                    image = '%s%s' % (Food.host[:-1], image)

                doc = OrderedDict([
                    ('id', id_),
                    ('building_id', building_id),
                    ('name', name),
                    ('short_name', short_name),
                    ('description', desc),
                    ('url', url),
                    ('tags', tags),
                    ('image', image),
                    ('campus', campus.upper()),
                    ('lat', lat),
                    ('lng', lng),
                    ('address', address),
                    ('hours', hours)
                ])

                Scraper.logger.info('Scraped %s @ %s.' % (
                    name,
                    campus.upper()
                ))

                Scraper.save_json(doc, location, id_)

        Scraper.logger.info('Food completed.')

    @staticmethod
    def get_hours(food_id):
        """Parse and return the restaurant's opening and closing times."""

        def conv_time(t):
            """Convert time of form "HH:MM p.d." to decimal (p.d. is one
            of a.m./p.m.)"""

            time, period = t[:-4].strip(), t[-4:].strip()

            # for mistyped times (i.e. http://map.utoronto.ca/json/hours/1329)
            if t[0] == ':':
                time = time[1:len(time)-2] + ':' + time[-2:]

            m = 0
            if ':' in time:
                h, m = [int(x) for x in time.split(':')]
            else:
                h = int(time)

            h += 12 if period == 'p.m.' else 0
            return h + (m / 60)

        headers = {
            'Referer': Food.host
        }
        html = Food.s.get('%s%s%s' % (Food.host, 'json/hours/', food_id),
            headers=headers).text
        soup = BeautifulSoup(html, 'html.parser')

        hours = OrderedDict()

        if not soup.find('tbody').text == '':
            days = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday',
                    'friday', 'saturday']

            timings = soup.find('tbody').find_all('td')

            for i in range(len(timings)):
                is_closed = True
                open_ = close = 0
                day, timing = days[i], timings[i].text

                if 'closed' not in timing:
                    is_closed = False
                    # timing is of form "HH:MM p.d. -HH:MM p.d."
                    open_, close = [conv_time(t) for t in timing.split(' -')]

                hours.update({day: OrderedDict([('closed', is_closed),
                                                ('open', open_),
                                                ('close', close)])})
        return hours



import requests
import json
from . import Scraper


class LayersScraper:
    """A superclass for scraping Layers of the UofT Map.

    Map is located at http://map.utoronto.ca
    """

    host = 'http://map.utoronto.ca/'
    s = requests.Session()

    @staticmethod
    def get_layers_json(campus):
        """Retrieve the JSON structure from host."""

        Scraper.logger.info('Retrieving map layers for %s.' % campus.upper())

        headers = {
            'Referer': LayersScraper.host
        }
        html = LayersScraper.s.get('%s%s%s' % (
            LayersScraper.host,
            'data/map/',
            campus
        ), headers=headers).text

        data = json.loads(html)
        return data['layers']

    @staticmethod
    def get_value(entry, val, number=False):
        """Retrieve the desired value from the parsed response dictionary."""

        if val in entry.keys():
            return entry[val]
        else:
            return 0 if number else ''



from ..utils import Scraper
from bs4 import BeautifulSoup
from datetime import datetime
from collections import OrderedDict
import json
import requests


class UTMAthletics:
    """A scraper for the UTM athletics schedule.

    The schedule is located at http://www.utm.utoronto.ca/athletics/schedule
    """

    host = 'http://www.utm.utoronto.ca/athletics/schedule/month/'

    @staticmethod
    def scrape(location='.', month=None):
        """Update the local JSON files for this scraper."""
        month = month or UTMAthletics.get_month(month)

        Scraper.logger.info('UTMAthletics initialized.')
        html = Scraper.get_html('%s%s' % (UTMAthletics.host, month))
        soup = BeautifulSoup(html, 'html.parser')

        athletics = OrderedDict()

        calendar = soup.find('div', class_='month-view')
        for tr in calendar.find_all('tr', class_='single-day'):
            for td in tr.find_all('td'):
                date = td.get('data-date')

                if not UTMAthletics.date_in_month(date, month):
                    continue

                events = []
                for item in td.find(class_='inner').find_all(class_='item'):

                    # event cancelled or athletic center closed
                    if item.find(class_='cancelled-item'):
                        continue

                    if item.find(class_='athletics-calendar-note'):
                        continue

                    title = item.find(class_='athletics-calendar-title').text
                    location_ = item.find(class_='athletics-calendar-location').text
                    start = item.find(class_='date-display-start').get('content')
                    end = item.find(class_='date-display-end').get('content')

                    events.append(OrderedDict([
                        ('title', title),
                        ('location', location_),
                        ('building_id', '332'),
                        ('start_time', start),
                        ('end_time', end)
                    ]))

                athletics[date] = OrderedDict([
                    ('date', date),
                    ('events', events)
                ])

        for date, doc in athletics.items():
            Scraper.save_json(doc, location, date)

        Scraper.logger.info('UTMAthletics completed.')

    @staticmethod
    def get_month(m):
        now = datetime.now()
        return '%s-%s' % (now.year, now.month)

    @staticmethod
    def date_in_month(d, m):
        d = datetime.strptime(d, '%Y-%m-%d')
        m = datetime.strptime(m, '%Y-%m')

        return d.month == m.month



from ..utils import Scraper
from bs4 import BeautifulSoup
from collections import OrderedDict
import json
import requests


class UTSGAthletics:

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('UTSGAthletics initialized.')
        Scraper.logger.info('Not implemented.')
        Scraper.logger.info('UTSGAthletics completed.')



from ..utils import Scraper
from bs4 import BeautifulSoup
from collections import OrderedDict
import json
import os
import requests


class UTSCTimetable:

    host = 'http://www.utsc.utoronto.ca/~registrar/scheduling/timetable'

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('UTSCTimetable initialized.')
        Scraper.logger.info('Not implemented.')
        Scraper.logger.info('UTSCTimetable completed.')



from distutils.core import setup
from codecs import open

setup(
    name = 'uoftscrapers',
    packages = [
        'uoftscrapers',
        'uoftscrapers.scrapers.utils',
        'uoftscrapers.scrapers.courses',
        'uoftscrapers.scrapers.buildings',
        'uoftscrapers.scrapers.textbooks',
        'uoftscrapers.scrapers.food',
        'uoftscrapers.scrapers.calendar',
        'uoftscrapers.scrapers.timetable',
        'uoftscrapers.scrapers.exams',
        'uoftscrapers.scrapers.athletics'
    ],
    version = '0.4.1',
    description = 'University of Toronto public web scraping scripts.',
    author = 'Qasim Iqbal',
    author_email = 'me@qas.im',
    url = 'https://github.com/cobalt-io/uoft-scrapers',
    download_url = 'https://github.com/cobalt-uoft/uoft-scrapers/tarball/0.4.1',
    package_data = {'': ['LICENSE.md']},
    package_dir = {'uoftscrapers': 'uoftscrapers'},
    include_package_data = True,
    keywords = ['uoft', 'scraper', 'toronto', 'university of toronto', 'cobalt'],
    install_requires = ['requests>=2.6.2', 'beautifulsoup4>=4.3.2', 'pytidylib>=0.2.4', 'lxml>=3.6.0', 'pytz>=2016.3'],
    classifiers = []
)



from ..utils import Scraper
from bs4 import BeautifulSoup
from datetime import datetime, date
from collections import OrderedDict
import requests
import pytz


class UTSGExams:
    """A scraper for UTSG exams.

    Data is scraped from http://www.artsci.utoronto.ca/current/exams/
    """

    host = 'http://www.artsci.utoronto.ca/current/exams/'
    s = requests.Session()

    @staticmethod
    def scrape(location='.', year=None):
        """Update the local JSON files for this scraper."""

        Scraper.logger.info('UTSGExams initialized.')

        exams = OrderedDict()

        for p in UTSGExams.get_exam_periods(year):
            Scraper.logger.info('Scraping %s exams.' % p.upper())

            headers = {
                'Referer': UTSGExams.host
            }
            html = UTSGExams.s.get('%s%s' % (UTSGExams.host, p),
                                   headers=headers).text
            soup = BeautifulSoup(html, 'html.parser')

            if not soup.find('table', class_='vertical listing'):
                # no exam data available
                Scraper.logger.info('No %s exams.' % p.upper())
                continue

            rows = soup.find('table', class_='vertical listing').find_all('tr')
            for row in rows[1:]:
                data = [x.text.strip() for x in row.find_all('td')]

                id_, course_id, course_code = UTSGExams.parse_course_info(p, data[0])

                if id_ is None:
                    continue

                section, location_ = data[1], data[4]
                date_ = UTSGExams.parse_date(data[2], p[-2:]) or ''
                start, end = UTSGExams.parse_time(data[3], date_) or (0, 0)

                doc = OrderedDict([
                    ('id', id_),
                    ('course_id', course_id),
                    ('course_code', course_code),
                    ('period', p.upper()),
                    ('date', date_),
                    ('start_time', start),
                    ('end_time', end),
                    ('sections', [])
                ])

                if id_ not in exams:
                    exams[id_] = doc

                exams[id_]['sections'].append(OrderedDict([
                    ('section', section),
                    ('location', location_)
                ]))

        for id_, doc in exams.items():
            Scraper.save_json(doc, location, id_)

        Scraper.logger.info('UTSGExams completed.')

    @staticmethod
    def parse_course_info(period, course_code):

        month, year, season = period[:-2], period[-2:], course_code[-1]

        endings = {
            'dec': {
                'F': '%s9' % str(int(year)),
                'Y': '%s9' % str(int(year) - 1)
            },
            'apr': {
                'S': '%s1' % str(year),
                'Y': '%s9' % str(int(year) - 1)
            },
            'june': {
                'F': '%s5F' % str(year),
                'Y': '%s5' % str(year)
            },
            'aug': {
                'S': '%s5S' % str(year),
                'Y': '%s5' % str(year)
            }
        }

        exam_id = course_id = None
        if month in endings and season in endings[month]:
            course_id = '%s20%s' % (course_code, endings[month][season])
            exam_id = '%s%s' % (course_id, period.upper())

        return exam_id, course_id, course_code

    @staticmethod
    def parse_date(date_, year):
        """Convert date of form `D DD MMM` to ISO 8601 format."""

        date_ = date_.split(' ')
        if len(date_) == 3:
            day, date_, month = date_

            return datetime.strptime('%s %s %s %s' % (day, date_, month, year),
                                     '%a %d %b %y').date().isoformat()

    @staticmethod
    def parse_time(time, d):
        """Convert time from `pd HH:MM - HH:MM` to start & end datetime."""

        def convert_time(t, is_pm=False):
            """Convert time from `HH:MM` to an ISO 8601 datetime."""
            h, m = [int(x) for x in t.split(':')]
            h += 12 if is_pm else 0

            date_ = datetime.strptime('%s %s %s' % (d, h, m), '%Y-%m-%d %H %M')
            return date_.replace(tzinfo=pytz.timezone('US/Eastern')).isoformat()

        time = list(filter(None, time.replace('-', '').split(' ')))
        if len(time) == 3:
            period, start, end = time
            after_12 = period == 'PM' or period == 'EV'

            return convert_time(start, after_12), convert_time(end, after_12)

    @staticmethod
    def get_exam_periods(year):
        if not year:
            year = date.today().year

        periods = []
        for m in ('dec', 'apr', 'june', 'aug'):
            y = year if not m == 'dec' else int(year) - 1
            periods.append('%s%s' % (m, str(y)[2:]))

        return periods



from ..utils import Scraper
from bs4 import BeautifulSoup
from datetime import datetime
from collections import OrderedDict
import requests
import pytz
import re


class UTMExams:
    """A scraper for UTM exams."""

    host = 'https://m.utm.utoronto.ca/'
    s = requests.Session()

    @staticmethod
    def scrape(location='.'):
        Scraper.logger.info('UTMExams initialized.')

        depts = UTMExams.get_page_links('list_dept.php?type=2')
        Scraper.logger.info('Got departments (1/3).')

        courses = []
        for dept in depts:
            courses.extend(UTMExams.get_page_links(dept))
        Scraper.logger.info('Got courses (2/3).')

        exams = UTMExams.retrieve_exams(courses)
        Scraper.logger.info('Got exams (3/3).')

        for id_, doc in exams.items():
            Scraper.save_json(doc, location, id_)

        Scraper.logger.info('UTMExams completed.')

    @staticmethod
    def retrieve_exams(courses):

        exams = OrderedDict()

        for course in courses:
            headers = {
                'Referer': UTMExams.host
            }
            html = UTMExams.s.get('%s%s' % (UTMExams.host, course),
                                  headers=headers).text
            soup = BeautifulSoup(html, 'html.parser')

            course_code = soup.find('div', class_='title').text.strip()

            # some course names include lecture code (see CHI200Y5Y)
            if ' ' in course_code:
                course_code, lecture_code = course_code.split(' ')
            else:
                lecture_code = None

            data = [br.previous_sibling.string.strip()
                    for br in soup.find('div', class_='info').find_all('br')]

            date = data[0].split(': ')[1]

            id_, course_id = UTMExams.get_course_id(course_code, date)

            period = UTMExams.get_period(date)

            if not id_ or not period:
                continue

            start, end = UTMExams.parse_time(data[1].split(': ')[1],
                                             data[2].split(': ')[1], date)

            sections = [UTMExams.parse_sections(room.split(': ')[1])
                        for room in [x for x in data[3:] if 'Room:' in x]]

            # append lecture code to section range if it exists
            if lecture_code:
                sections[0]['section'] = '%s %s' % (lecture_code,
                                                    sections[0]['section'])

            doc = OrderedDict([
                ('id', id_),
                ('course_id', course_id),
                ('course_code', course_code),
                ('period', period),
                ('date', date),
                ('start_time', start),
                ('end_time', end),
                ('sections', [])
            ])

            if id_ not in exams:
                exams[id_] = doc

            for section in sections:
                exams[id_]['sections'].append({
                    'section': section['section'].strip(),
                    'locaton': section['room']
                })
        return exams

    @staticmethod
    def get_page_links(endpoint):
        headers = {
            'Referer': UTMExams.host
        }
        html = UTMExams.s.get('%s%s' % (UTMExams.host, endpoint),
                              headers=headers).text
        soup = BeautifulSoup(html, 'html.parser')
        return [li.find('a')['href']
                for li in soup.find('ul', class_='link').find_all('li')]

    @staticmethod
    def get_period(d):
        def get_date(month, date, year):
            months = {
                'dec': 12,
                'apr': 4,
                'june': 6,
                'aug': 8
            }
            return datetime.strptime('%s-%d-%d' % (year, months[month], date),
                                     '%Y-%m-%d')

        d = datetime.strptime(d, '%Y-%m-%d')

        year = d.year
        month = None

        for m, ld in (('dec', 31), ('apr', 30), ('june', 30), ('aug', 31)):
            if get_date(m, 1, year) <= d <= get_date(m, ld, year):
                month = m
                break

        if month:
            return '%s%s' % (month.upper(), str(year)[2:])

    @staticmethod
    def get_course_id(course_code, date):
        d = datetime.strptime(date, '%Y-%m-%d')
        month, year, period = d.strftime("%b").lower(), d.year, UTMExams.get_period(date)
        endings = {
            'dec': {
                'F': '%s9' % str(year),
                'Y': '%s9' % str(int(year) - 1)
            },
            'apr': {
                'S': '%s1' % str(year),
                'Y': '%s9' % str(int(year) - 1)
            },
            'june': {
                'F': '%s5F' % str(year),
                'Y': '%s5' % str(year)
            },
            'aug': {
                'S': '%s5S' % str(year),
                'Y': '%s5' % str(year)
            }
        }

        season = course_code[-1]
        exam_id = course_id = None

        if month in endings and season in endings[month]:
            course_id = '%s%s' % (course_code, endings[month][season])
            exam_id = '%s%s' % (course_id, period)

        return exam_id, course_id

    @staticmethod
    def parse_sections(room):
        section = ''
        if '(' in room:
            room, section = [x.strip()
                             for x in re.sub('[()]', ' ', room).split('  ')]
        return {'section': section, 'room': room}

    @staticmethod
    def parse_time(start, end, date):
        def convert_time(t):
            h, m, s = [int(x) for x in t.split(':')]
            d = datetime.strptime('%s %s %s %s' % (date, h, m, s), '%Y-%m-%d %H %M %S')
            return d.replace(tzinfo=pytz.timezone('US/Eastern')).isoformat()

        return convert_time(start), convert_time(end)
